
# Copyright (C) 2015 Centro de Computacao Cientifica e Software Livre
# Departamento de Informatica - Universidade Federal do Parana
#
# This file is part of portalmec.
#
# portalmec is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# portalmec is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with portalmec.  If not, see <http://www.gnu.org/licenses/>.

class V1::ComplaintVotesController < ApplicationController
    include ::DeletedObjectsController
    include ::Paginator
    before_action :set_complaint_vote, only: [:show, :update, :destroy]

    # GET /v1/complaint_votes
    def index
        complaint_votes = paginate ComplaintVote
        render json: complaint_votes
    end

    # GET /v1/complaint_votes/{id}
    def show
        render json: @complaint_vote
    end

    # POST /v1/complaint_votes
    def create
        @complaint_vote = ComplaintVote.new(complaint_vote_params)

        if @complaint_vote.save
            render json: @complaint_vote, status: :created
        else
            render json: @complaint_vote.errors, status: :unprocessable_entity
        end
        
        verify_votes(@complaint_vote)
    end

    # Set the params required for creating a complaint vote
    def complaint_vote_params
        params.require(:complaint_vote).permit(:complainable_type, :complainable_id, :pros, :cons, :user_id)
    end

    # PUT/PATCH /v1/complaint_votes/{id}
    def update
        if @complaint_vote.update(complaint_vote_params)
            render json: @complaint_vote
        else 
            render json: @complaint_vote.errors, status: :unprocessable_entity
        end
    end

    # DELETE /v1/complaint_votes/{id}
    def destroy
        @complaint_vote.destroy
        response = { 'status': 'deleted' }
        render status: :ok, json: response
    end

    def set_complaint_vote
        # @complaint_vote = ComplaintVote.where(id: params[:id]).first
        
        # render status: :not_found if @complaint_vote.blank?
        
        # @complaint_vote

        @complaint_vote ||= ComplaintVote.find_by_id(params[:id]).first
        if @complaint_vote.blank?
            render status: :not_found
        end
    end
    
    # if the difference between the pros and cons votes is more than 2, destroy the learning object  
    def verify_votes(complaint_vote)

        data = ComplaintVote.select("complainable_id, sum(pros) as pros, sum(cons) as cons")
                            .where("complainable_id = ?", complaint_vote.complainable_id)
                            .group("complaint_votes.complainable_id")

        data.each do |d|
            if (d.complainable_id == complaint_vote.complainable_id)
                if d.pros - d.cons > 2
                    ComplaintVote.where("complainable_id = ?", complaint_vote.complainable_id).destroy_all
                    Complaint.where("complainable_id = ?", complaint_vote.complainable_id).destroy_all
                    LearningObject.where(id: complaint_vote.complainable_id).update_all(deleted_at: Time.now)
                    #Complaint.find_by_complainable_id(complaint_vote.complainable_id).update(state: 2)
                    #Complaint.find_by_complainable_id(complaint_vote.complainable_id).destroy
                    #ComplaintVote.find_by_complainable_id(complaint_vote.complainable_id).destroy
                    #LearningObject.find_by_id(complaint_vote.complainable_id).destroy
                    # puts "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-"
                    # puts "  pros maior que cons, pros = #{d.pros}, cons = #{d.cons}"
                    # puts "=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-"
                elsif d.cons - d.pros > 2
                    ComplaintVote.where("complainable_id = ?", complaint_vote.complainable_id).destroy_all
                    Complaint.where("complainable_id = ?", complaint_vote.complainable_id).update_all(state: Complaint.states[:rejected])
                    LearningObject.where(id: complaint_vote.complainable_id).update_all(state: LearningObject.states[:published])
                end
            end
        end
    end

end
